--
-- KroneSwadro1010
-- Specialization for KroneSwadro1010
--
-- @author Felix "Outlaw" Sorge
-- @date  23/06/09
-- @web www.ls-modsource.de - www.ls-mods.de
--
-- Copyright (C) Outlaw, All Rights Reserved.
--

KroneSwadro1010 = {};

function KroneSwadro1010.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Trailer, specializations);
end;

function KroneSwadro1010:load(xmlFile)
	self.lowerRmp = SpecializationUtil.callSpecializationsFunction("lowerRmp");
	self.row = SpecializationUtil.callSpecializationsFunction("row");
	self.connectCollisions = SpecializationUtil.callSpecializationsFunction("connectCollisions");
	self.anim = SpecializationUtil.callSpecializationsFunction("anim");
	self.Go = {}; 
	self.Done = {};
	self.charId = {};
	self.clipIndex = {};
	self.CheckDone = {};
	self.moveColli = {}; 
    self.animParts = {};
	self.collisionArm = {};
	self.startpoint = {};
	self.loopCheck = {};
	self.Speed = {};
	local count = getXMLInt(xmlFile, "vehicle.animParts#count");
	local part = self.animParts;
    for i=1, count do
		part[i] = {};
        local partname = string.format("vehicle.animParts.part".."%d", i);
	    local nameR = getXMLString(xmlFile, partname.."#name");
		self.charId[nameR] = Utils.indexToObject(self.components, getXMLString(xmlFile, partname.."#rootNode"));
		self.clipIndex[nameR] = getXMLString(xmlFile, partname.."#animationClip");
		self.CheckDone[nameR] = false;
		self.moveColli[nameR] = getXMLString(xmlFile, partname.."#moveColli");
		self.startpoint[nameR] = Utils.getNoNil(getXMLFloat(xmlFile, partname.."#startpoint"),0.0);
		if self.moveColli[nameR] ~= nil then
			local Collision = {};
			Collision.collision = Utils.indexToObject(self.components, getXMLString(xmlFile, partname.."#collision"));
			Collision.collisionAttacher = Utils.indexToObject(self.components, getXMLString(xmlFile, partname.."#collsionAttacher"));
			Collision.armAttacher = Utils.indexToObject(self.components, getXMLString(xmlFile, partname.. "#armAttacher"));
			Collision.index = 0;
			self.collisionArm[nameR] = Collision;
			self:connectCollisions(nameR);
		end; 
		local charId = getAnimCharacterSet(self.charId[nameR]);
		local clipIndex = getAnimClipIndex(charId, self.clipIndex[nameR]);
		assignAnimTrackClip(charId , 0, clipIndex);
		setAnimTrackTime(charId, 0, self.startpoint[nameR]);
		enableAnimTrack(charId, 0);
		disableAnimTrack(charId, 0);
		self.loopCheck[nameR] = false;
		self.Speed[nameR] = 1.0;
	end; 
			
	local numWindrowerDropAreas = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.windrowerDropAreas#count"), 0);
    if numWindrowerDropAreas ~= 1 or numWindrowerDropAreas ~= table.getn(self.cuttingAreas) then
        print("Warning: number of cutting areas and drop areas should be equal");
    end;
	
	self.windrowerDropAreas = {}
    for i=1, numWindrowerDropAreas do
        self.windrowerDropAreas[i] = {};
        local areanamei = string.format("vehicle.windrowerDropAreas.windrowerDropArea%d", i);
        self.windrowerDropAreas[i].start = Utils.indexToObject(self.components, getXMLString(xmlFile, areanamei .. "#startIndex"));
        self.windrowerDropAreas[i].width = Utils.indexToObject(self.components, getXMLString(xmlFile, areanamei .. "#widthIndex"));
        self.windrowerDropAreas[i].height = Utils.indexToObject(self.components, getXMLString(xmlFile, areanamei .. "#heightIndex"));
    end;

    local numCuttingAreas = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.cuttingAreas#count"), 0);
    for i=1, numCuttingAreas do
        local areanamei = string.format("vehicle.cuttingAreas.cuttingArea%d", i);
        self.cuttingAreas[i].foldMinLimit = Utils.getNoNil(getXMLFloat(xmlFile, areanamei .. "#foldMinLimit"), 0);
        self.cuttingAreas[i].foldMaxLimit = Utils.getNoNil(getXMLFloat(xmlFile, areanamei .. "#foldMaxLimit"), 1);
    end;
	
	local windrowerSound = getXMLString(xmlFile, "vehicle.windrowerSound#file");
    if windrowerSound ~= nil and windrowerSound ~= "" then
        windrowerSound = Utils.getFilename(windrowerSound, self.baseDirectory);
        self.windrowerSound = createSample("windrowerSound");
        self.windrowerSoundEnabled = false;
        loadSample(self.windrowerSound, windrowerSound, false);
        self.windrowerSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.windrowerSound#pitchOffset"), 1);
        self.windrowerSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.windrowerSound#volume"), 1.0);
    end;
	
	self.wheatParticleSystems = {};
    local i = 0;
    while true do
        local namei = string.format("vehicle.wheatParticleSystems.wheatParticleSystems(%d)", i);
		local nodei = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, namei .. "#index"));
		if nodei == nil then
			break;
		end; 
        Utils.loadParticleSystem(xmlFile, self.wheatParticleSystems, namei, nodei, false, nil, self.baseDirectory)		
		i = i +1;		
    end;
	
	self.barleyParticleSystems = {};
    local i = 0;
    while true do
        local namei = string.format("vehicle.barleyParticleSystems.barleyParticleSystems(%d)", i);
		local nodei = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, namei .. "#index"));
		if nodei == nil then
			break;
		end; 
        Utils.loadParticleSystem(xmlFile, self.barleyParticleSystems, namei, nodei, false, nil, self.baseDirectory)		
		i = i +1;		
    end;
	
	self.grassParticleSystems = {};
    local i = 0;
    while true do
        local namei = string.format("vehicle.grassParticleSystems.grassParticleSystems(%d)", i);
		local nodei = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, namei .. "#index"));
		if nodei == nil then
			break;
		end; 
        Utils.loadParticleSystem(xmlFile, self.grassParticleSystems, namei, nodei, false, nil, self.baseDirectory)		
		i = i +1;		
    end;
	
	self.drygrassParticleSystems = {};
    local i = 0;
    while true do
        local namei = string.format("vehicle.drygrassParticleSystems.drygrassParticleSystems(%d)", i);
		local nodei = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, namei .. "#index"));
		if nodei == nil then
			break;
		end; 
        Utils.loadParticleSystem(xmlFile, self.drygrassParticleSystems, namei, nodei, false, nil, self.baseDirectory)		
		i = i +1;		
    end;
	
	self.keyType = InputBinding.getButtonKeyName;
	self.keyName = "Taste"; 
		
end;

function KroneSwadro1010:update(dt)
	
	if self:getIsActive() then
	
		if self:getIsActiveForInput() and InputBinding.hasEvent(InputBinding.KroneSwadro1010_transport) and not self.turnOn and not self.CheckDone.lower and not self.Go.lower then
			self.Speed.rower1 = 1.0; 
			self.Speed.rower2 = 1.0; 
			self.Speed.rower3 = 1.0; 
			self.Speed.transport = 1.0; 
			self.Go.transport = not self.Go.transport;
			self.Done.transport = true; 
			if self.Go.transport then
				self.Go.rower1 = 8000;
				self.Done.rower1 = true; 
				self.Go.rower2 = 8000;
				self.Done.rower2 = true; 
				self.Go.rower3 = 8000;
				self.Done.rower3 = true; 
			else
				self.Go.rower1 = 2000; 
				self.Done.rower1 = true; 
				self.Go.rower2 = 4000; 
				self.Done.rower2 = true; 
				self.Go.rower3 = 2000; 
				self.Done.rower3 = true; 
			end; 	
		end;
	
		if self:getIsActiveForInput() and InputBinding.hasEvent(InputBinding.KroneSwadro1010_lower) and not self.CheckDone.transport and not self.Go.transport and self.turnOn then
			self.Go.lower = not self.Go.lower;
			self.Done.lower = true; 
			self.Speed.rower1 = 1.0; 
			self.Speed.rower2 = 2.0; 
			self.Speed.rower3 = 1.0; 
			if not self.Go.lower then
				self.Go.rower1 = 2000; 
				self.Done.rower1 = true; 
				self.Go.rower2 = 4000; 
				self.Done.rower2 = true; 
				self.Go.rower3 = 2000; 
				self.Done.rower3 = true; 
			else
				self.Go.rower1 = 0; 
				self.Done.rower1 = true; 
				self.Go.rower2 = 0; 
				self.Done.rower2 = true; 
				self.Go.rower3 = 0; 
				self.Done.rower3 = true; 
			
			end; 
		end;
		
		if self:getIsActiveForInput() and InputBinding.hasEvent(InputBinding.KroneSwadro1010_on) and not self.CheckDone.transport and not self.Go.transport and not self.CheckDone.lower and not self.Go.lower then
			self.turnOn = not self.turnOn;
			if self.turnOn then
				self.Go.row = true; 
				self.Done.row = true;
				self.loopCheck.row = true;
				self.Speed.row = 4.0; 
			else
				self.Go.row = false;
				self.Done.row = true;
			end;
		end;
		
		if self.fixAttacher then
			local implement = self.attacherVehicle:getImplementByObject(self);
			if implement ~= nil then
				local joint = self.attacherVehicle.attacherJoints[implement.jointDescIndex];
				self.saveJoint = self.attacherVehicle.attacherJoints[implement.jointDescIndex];
				self.maxRotJoint = joint.maxRotLimit;
				self.attacherJointIndex = implement.jointDescIndex;
				self.rotationCopy  = joint.minRot;
				joint.minRot = {Utils.degToRad(2),0,0};			
				joint.maxRotLimit = {0, 0, 0};
				setJointRotationLimit(joint.jointIndex, 2, true, 0, 0);
			end;
			self.fixAttacher = false;
		end;
		
		if not self.turnOn then
			if self.windrowerSoundEnabled then
				stopSample(self.windrowerSound);
				self.windrowerSoundEnabled = false;
			end; 
			self:lowerRmp(); 		
		else
			if not self.Go.lower then
				Utils.setEmittingState(self.drygrassParticleSystems, false)
				Utils.setEmittingState(self.grassParticleSystems, false)
				Utils.setEmittingState(self.wheatParticleSystems, false)
				Utils.setEmittingState(self.barleysParticleSystems, false)
			end; 
			if self.Go.lower and self.CheckDone.lower and not self.Done.lower then 
				self:row(FruitUtil.FRUITTYPE_BARLEY);
				self:row(FruitUtil.FRUITTYPE_WHEAT);
				self:row(FruitUtil.FRUITTYPE_GRASS);
				self:row(FruitUtil.FRUITTYPE_DRYGRASS);
			end; 
			if not self.windrowerSoundEnabled and self:getIsActiveForSound() then
				playSample(self.windrowerSound, 0, self.windrowerSoundVolume, 0);
	            setSamplePitch(self.windrowerSound, self.windrowerSoundPitchOffset);
	            self.windrowerSoundEnabled = true;
			end; 	
			if not self.backupRmp then
				self.backupRmp = self.attacherVehicle.motor.minRpm;
			end; 			
			if self.attacherVehicle.motor.minRpm >= -500 then
				self.attacherVehicle.motor.minRpm = self.attacherVehicle.motor.minRpm - 10; 
			end; 		
		end;
		
	end; 		
	
	if self.Go.rower1 == nil then
		self.Go.rower1 = 8000;
		self.Done.rower1 = true; 
		self.Go.rower2 = 8000;
		self.Done.rower2 = true; 
		self.Go.rower3 = 8000;
		self.Done.rower3 = true; 
		self.Speed.rower1 = 10.0;
		self.Speed.rower2 = 10.0;
		self.Speed.rower3 = 10.0;
		self.Speed.transport = 10.0; 
		self.Go.transport = not self.Go.transport;
		self.Done.transport = true; 
	end; 

	if self.Go.pillar ~= nil and self.Done.pillar ~= false then
		self:anim("pillar"); 
	end; 
	
	if self.Go.row ~= nil and self.Done.row ~= false then
		self:anim("row"); 
	end;

	if self.Go.rower1 ~= nil and self.Done.rower1 ~= false then
		self:anim("rower1"); 
	end;
	
	if self.Go.rower2 ~= nil and self.Done.rower2 ~= false then
		self:anim("rower2"); 
	end;
	
	if self.Go.rower3 ~= nil and self.Done.rower3 ~= false then
		self:anim("rower3"); 
	end;

	if self.Go.transport ~= nil and self.Done.transport ~= false then
		self:anim("transport"); 
	end;
	
	if self.Go.lower ~= nil and self.Done.lower ~= false then
		self:anim("lower"); 
	end;
		
end;

function KroneSwadro1010:draw()
	if not self.Go.transport and not self.turnOn and not self.CheckDone.lower and not self.Go.lower then
		g_currentMission:addExtraPrintText(self.keyName.. " " ..self.keyType(InputBinding.KroneSwadro1010_transport).. ": " ..g_i18n:getText("KroneSwadro1010_3")); 
	elseif not self.turnOn and not self.CheckDone.lower and not self.Go.lower then
		g_currentMission:addExtraPrintText(self.keyName.. " " ..self.keyType(InputBinding.KroneSwadro1010_transport).. ": " ..g_i18n:getText("KroneSwadro1010_4")); 
	end; 
	
	if self.turnOn and not self.CheckDone.transport and not self.Go.transport and not self.CheckDone.lower and not self.Go.lower then
		g_currentMission:addExtraPrintText(self.keyName.. " " ..self.keyType(InputBinding.KroneSwadro1010_on).. ": " ..g_i18n:getText("KroneSwadro1010_2")); 
	elseif not self.CheckDone.transport and not self.Go.transport and not self.CheckDone.lower and not self.Go.lower then
		g_currentMission:addExtraPrintText(self.keyName.. " " ..self.keyType(InputBinding.KroneSwadro1010_on).. ": " ..g_i18n:getText("KroneSwadro1010_1")); 
	end; 
	
	if self.Go.lower and not self.CheckDone.transport and not self.Go.transport and self.turnOn then
		g_currentMission:addExtraPrintText(self.keyName.. " " ..self.keyType(InputBinding.KroneSwadro1010_lower).. ": " ..g_i18n:getText("KroneSwadro1010_6")); 
	elseif not self.CheckDone.transport and not self.Go.transport and self.turnOn then 
		g_currentMission:addExtraPrintText(self.keyName.. " " ..self.keyType(InputBinding.KroneSwadro1010_lower).. ": " ..g_i18n:getText("KroneSwadro1010_5")); 
	end; 
end;

function KroneSwadro1010:anim(varName, loopCheck)
	if self.moveColli[varName] ~= nil then
		local Collision = self.collisionArm[varName];
		setJointFrame(Collision.index, 0, Collision.armAttacher);
	end;
	local loopCheck = self.loopCheck[varName];
	local speed = Utils.getNoNil(self.Speed[varName],1);
	local offset = 10;  
	local charId = getAnimCharacterSet(self.charId[varName]);
	local clipIndex = getAnimClipIndex(charId, self.clipIndex[varName]);
	if self.Done[varName] ~= false then
		if self.Go[varName] == true then
			assignAnimTrackClip(charId , 0, clipIndex);
			setAnimTrackLoopState(charId, 0, loopCheck);
			setAnimTrackSpeedScale(charId, 0, speed);
			enableAnimTrack(charId, 0);
			if getAnimTrackTime(charId, 0) >= getAnimClipDuration(charId, clipIndex) and loopCheck == false then
				self.Done[varName] = false;	
				self.CheckDone[varName] = self.Go[varName];	
			end;
			if getAnimTrackTime(charId, 0) >= getAnimClipDuration(charId, clipIndex) and loopCheck then
				local setTime = getAnimClipDuration(charId, clipIndex) - getAnimClipDuration(charId, clipIndex);
				setAnimTrackTime(charId, 0, setTime);
			end;
		elseif self.Go[varName] == false then
			if loopCheck == true then
				self.Done[varName] = false;	
			else
				assignAnimTrackClip(charId , 0, clipIndex);
				setAnimTrackLoopState(charId, 0, loopCheck);
				setAnimTrackSpeedScale(charId, 0, -speed);
				enableAnimTrack(charId, 0);
				if getAnimTrackTime(charId, 0) <= 0 then
					self.Done[varName] = false;	
					self.CheckDone[varName] = self.Go[varName];
				end;
			end;
		elseif self.Go[varName] ~= nil then
			assignAnimTrackClip(charId , 0, clipIndex);
			setAnimTrackLoopState(charId, 0, loopCheck);
			if self.Go[varName] < 0.0 then
				self.Go[varName] = 0.0;
			elseif self.Go[varName] > getAnimClipDuration(charId, clipIndex) then
				self.Go[varName] = getAnimClipDuration(charId, clipIndex);
			end;
			if getAnimTrackTime(charId, 0) >= self.Go[varName] then
				setAnimTrackSpeedScale(charId, 0, -speed);
			else
				setAnimTrackSpeedScale(charId, 0, speed);
			end;
			enableAnimTrack(charId, 0);
			if getAnimTrackTime(charId, 0) <= self.Go[varName] + offset and getAnimTrackTime(charId, 0) >= self.Go[varName] - offset then
				self.Done[varName] = false;	
				self.CheckDone[varName] = self.Go[varName];
			end;
		end;
	end;
	if self.Done[varName] == false then
		disableAnimTrack(charId, 0);
	end;
end; 

function KroneSwadro1010:connectCollisions(varName)
	local Collision = self.collisionArm[varName];
	local constr = JointConstructor:new();
	constr:setActors(self.rootNode, Collision.collision);
	constr:setJointTransforms(Collision.armAttacher, Collision.collisionAttacher);
	for i=1, 3 do		            
		constr:setRotationLimit(i-1, 0, 0, 0);
		constr:setTranslationLimit(i-1, true, 0, 0);
	end;
	Collision.index = constr:finalize();
end;

function KroneSwadro1010:mouseEvent(posX, posY, isDown, isUp, button)
end;

function KroneSwadro1010:keyEvent(unicode, sym, modifier, isDown)
	if isDown and sym==Input.KEY_8 then
		self.keySet = not self.keySet; 
		if self.keySet then
			self.keyType = InputBinding.getButtonName;
			self.keyName = "Button"; 
		else
			self.keyType = InputBinding.getButtonKeyName;
			self.keyName = "Taste"; 
		end; 
	end;
end;

function KroneSwadro1010:onAttach()
	self.fixAttacher = true; 
	self.Go.pillar = not self.Go.pillar; 
	self.Done.pillar = true; 
end; 	

function KroneSwadro1010:onDetach()
	local joint = self.saveJoint; 
	joint.minRot = self.rotationCopy;
	joint.maxRotLimit = self.maxRotJoint;
	self.Go.pillar = not self.Go.pillar; 
	self.Done.pillar = true; 
	Utils.setEmittingState(self.drygrassParticleSystems, false)	
	Utils.setEmittingState(self.grassParticleSystems, false)
	Utils.setEmittingState(self.wheatParticleSystems, false)
	Utils.setEmittingState(self.barleysParticleSystems, false)
	if self.windrowerSoundEnabled then
		stopSample(self.windrowerSound);
		self.windrowerSoundEnabled = false;
	end; 
	self:lowerRmp();
end;

function KroneSwadro1010:delete()
	self:lowerRmp();
	if self.windrowerSoundEnabled then
		stopSample(self.windrowerSound);
		self.windrowerSoundEnabled = false;
	end; 
	Utils.setEmittingState(self.drygrassParticleSystems, false)
	Utils.setEmittingState(self.grassParticleSystems, false)
	Utils.setEmittingState(self.wheatParticleSystems, false)
	Utils.setEmittingState(self.barleysParticleSystems, false)	
end;

function KroneSwadro1010:getSaveAttributesAndNodes(nodeIdent)
	local attributes = 'trsp="'..tostring(self.CheckDone.transport)..'"';
	return attributes, nil;
end;

function KroneSwadro1010:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
    local trsp = getXMLBool(xmlFile, key.."#trsp");
	if trsp and not resetVehicles then
		self.Go.rower1 = 8000;
		self.Done.rower1 = true; 
		self.Go.rower2 = 8000;
		self.Done.rower2 = true; 
		self.Go.rower3 = 8000;
		self.Done.rower3 = true; 
		self.Speed.rower1 = 10.0;
		self.Speed.rower2 = 10.0;
		self.Speed.rower3 = 10.0;
		self.Speed.transport = 10.0; 
		self.Go.transport = not self.Go.transport;
		self.Done.transport = true; 
	elseif not resetVehicles then
		self.Go.rower1 = 2000;
		self.Done.rower1 = true; 
		self.Go.rower2 = 4000;
		self.Done.rower2 = true; 
		self.Go.rower3 = 2000;
		self.Done.rower3 = true; 
		self.Speed.rower1 = 10.0;
		self.Speed.rower2 = 10.0;
		self.Speed.rower3 = 10.0;
		self.Speed.transport = 10.0; 
	end; 
end;

function KroneSwadro1010:row(fruitTypeR)
	local numDropAreas = table.getn(self.windrowerDropAreas);
	local numAreas = 3; 
    local sum = 0;
    local fruitType = fruitTypeR;	 
    local fruitTypeFix = false;
    local foldAnimTime = self.foldAnimTime;
    for i=1, numAreas do
		local cuttingArea = self.cuttingAreas[i];
	
        if cuttingArea then
            local x,y,z = getWorldTranslation(cuttingArea.start);
            local x1,y1,z1 = getWorldTranslation(cuttingArea.width);
            local x2,y2,z2 = getWorldTranslation(cuttingArea.height);
			local ratio = g_currentMission.windrowCutLongRatio;
			if not fruitTypeFix then
                fruitType = fruitTypeR;
            end;
			local area = Utils.updateFruitCutLongArea(fruitType, x, z, x1, z1, x2, z2, 0)*ratio;
            area = area + Utils.updateFruitWindrowArea(fruitType, x, z, x1, z1, x2, z2, 0);
            if area == 0 and not fruitTypeFix then
				fruitType = fruitTypeR; 
                area = Utils.updateFruitCutLongArea(fruitType, x, z, x1, z1, x2, z2, 0)*ratio;
                area = area + Utils.updateFruitWindrowArea(fruitType, x, z, x1, z1, x2, z2, 0);
				if fruitTypeR == FruitUtil.FRUITTYPE_DRYGRASS then
					Utils.setEmittingState(self.drygrassParticleSystems, false)
				elseif fruitTypeR == FruitUtil.FRUITTYPE_GRASS then	
					Utils.setEmittingState(self.grassParticleSystems, false)
				elseif fruitTypeR == FruitUtil.FRUITTYPE_WHEAT then
					Utils.setEmittingState(self.wheatParticleSystems, false)
				elseif fruitTypeR == FruitUtil.FRUITTYPE_BARLEY then 	
					Utils.setEmittingState(self.barleyParticleSystems, false)
				end; 
            end;
			if area > 0 then
                fruitTypeFix = true;
            end;
			if numDropAreas >= numAreas then
                if area > 0 then
					local dropArea = self.windrowerDropAreas[i];
					if fruitTypeR == FruitUtil.FRUITTYPE_DRYGRASS then
						Utils.setEmittingState(self.drygrassParticleSystems, true)
					elseif fruitTypeR == FruitUtil.FRUITTYPE_GRASS then	
						Utils.setEmittingState(self.grassParticleSystems, true)
					elseif fruitTypeR == FruitUtil.FRUITTYPE_WHEAT then
						Utils.setEmittingState(self.wheatParticleSystems, true)
					elseif fruitTypeR == FruitUtil.FRUITTYPE_BARLEY then 	
						Utils.setEmittingState(self.barleyParticleSystems, true)
					end; 
                    local x,y,z = getWorldTranslation(dropArea.start);
                    local x1,y1,z1 = getWorldTranslation(dropArea.width);
                    local x2,y2,z2 = getWorldTranslation(dropArea.height);
                    local old, total = Utils.getFruitWindrowArea(fruitType, x, z, x1, z1, x2, z2);
                    area = area + old;
                    local value = area / total;
                    if value < 1 and value > 0.08 then
                         value = 1;
                    else
                        value = math.floor(value + 0.6); -- round, biased to the bigger value
                    end;
                    if value >= 1 then
                        value = math.min(value, g_currentMission.maxWindrowValue);
                        Utils.updateFruitWindrowArea(fruitType, x, z, x1, z1, x2, z2, value, true);
                    end;
                end;
            else
                 sum = sum + area;
            end;
        end;
    end;
    if sum > 0 and numDropAreas > 0 then
	    local dropArea = self.windrowerDropAreas[1];
	    local x,y,z = getWorldTranslation(dropArea.start);
	    local x1,y1,z1 = getWorldTranslation(dropArea.width);
	    local x2,y2,z2 = getWorldTranslation(dropArea.height);
	    local old, total = Utils.getFruitWindrowArea(fruitType, x, z, x1, z1, x2, z2);
	    sum = sum + old;
	    local value = math.floor(sum / total + 0.7); -- round, biased to the bigger value
	    if value >= 1 then
	        value = math.min(value, g_currentMission.maxWindrowValue);
	        Utils.updateFruitWindrowArea(fruitType, x, z, x1, z1, x2, z2, value, true);
	    end;
    end;
end; 

function KroneSwadro1010:lowerRmp()
	if self.backupRmp and self.attacherVehicle then
		if self.attacherVehicle.motor.minRpm <= self.backupRmp then
			self.attacherVehicle.motor.minRpm = self.attacherVehicle.motor.minRpm + 10; 
		else
			self.attacherVehicle.motor.minRpm = self.backupRmp; 
			self.backupRmp = nil; 			
		end; 	
	end; 
end;

function KroneSwadro1010:onLeave()
	if self.windrowerSoundEnabled then
		stopSample(self.windrowerSound);
		self.windrowerSoundEnabled = false;
	end; 
	Utils.setEmittingState(self.drygrassParticleSystems, false)
	Utils.setEmittingState(self.grassParticleSystems, false)
	Utils.setEmittingState(self.wheatParticleSystems, false)
	Utils.setEmittingState(self.barleysParticleSystems, false)
end; 